@@ -9,8 +9,10 @@ use std::sync::Mutex;
 use editor::ImgEditor;
 use glycin_utils::image_rs::Handler;
 use glycin_utils::*;
+use gufo_common::cicp::Cicp;
 use image::{codecs, AnimationDecoder, ImageDecoder, ImageResult, Limits};
 use log::trace;
+use memory_format::MemoryFormatInfo;
 
 init_main_loader_editor!(ImgDecoder::default(), ImgEditor::default());
 
@@ -22,6 +24,7 @@ type FrameSender = Sender<Result<Frame, ProcessError>>;
 pub struct ImgDecoder {
     pub format: Mutex<Option<ImageRsFormat<Reader>>>,
     pub thread: Mutex<Option<(std::thread::JoinHandle<()>, FrameReceiver)>>,
+    pub cicp: Mutex<Option<Cicp>>,
 }
 
 fn animated_worker(
@@ -166,12 +169,34 @@ impl LoaderImplementation for ImgDecoder {
         }
         let mut image_info = format.info();
 
-        let exif = exif::Reader::new().read_from_container(&mut data.clone());
-        image_info.details.exif = exif
-            .ok()
-            .map(|x| BinaryData::from_data(x.buf()))
-            .transpose()
-            .expected_error()?;
+        // TODO: Unnecessary clone of data
+        let metadata = gufo::RawMetadata::for_guessed(data.get_ref().to_vec());
+
+        if let Ok(metadata) = metadata {
+            image_info.details.exif = metadata
+                .exif
+                .first()
+                .map(BinaryData::from_data)
+                .transpose()
+                .expected_error()?;
+
+            image_info.details.xmp = metadata
+                .xmp
+                .first()
+                .map(BinaryData::from_data)
+                .transpose()
+                .expected_error()?;
+        }
+
+        // TODO: Unnecessary clone of data
+        let gufo_image = gufo::Image::new(data.into_inner());
+        let data = Cursor::new(match gufo_image {
+            Ok(gufo_image) => {
+                *self.cicp.lock().unwrap() = gufo_image.cicp();
+                gufo_image.into_inner()
+            }
+            Err(err) => err.into_inner(),
+        });
 
         if format.decoder.is_animated() {
             let (send, recv) = channel();
@@ -185,7 +210,7 @@ impl LoaderImplementation for ImgDecoder {
     }
 
     fn frame(&self, _frame_request: FrameRequest) -> Result<Frame, ProcessError> {
-        let frame = if let Some(decoder) = std::mem::take(&mut *self.format.lock().unwrap()) {
+        let mut frame = if let Some(decoder) = std::mem::take(&mut *self.format.lock().unwrap()) {
             decoder.frame().expected_error()?
         } else if let Some((ref thread, ref recv)) = *self.thread.lock().unwrap() {
             thread.thread().unpark();
@@ -194,6 +219,8 @@ impl LoaderImplementation for ImgDecoder {
             unreachable!()
         };
 
+        frame.details.cicp = self.cicp.lock().unwrap().map(|x| x.into());
+
         Ok(frame)
     }
 }
@@ -222,6 +249,13 @@ pub struct ImageRsFormat<T: std::io::BufRead + std::io::Seek> {
 impl ImageRsFormat<Reader> {
     fn create(data: Reader, mime_type: &str) -> Result<Self, ProcessError> {
         Ok(match mime_type {
+            "image/apng" => Self::new(ImageRsDecoder::Png(
+                codecs::png::PngDecoder::new(data).expected_error()?,
+            ))
+            .format_name("Animated PNG")
+            .supports_two_alpha_modes(true)
+            .supports_two_grayscale_modes(true)
+            .default_bit_depth(8),
             "image/bmp" => Self::new(ImageRsDecoder::Bmp(
                 codecs::bmp::BmpDecoder::new(data).expected_error()?,
             ))
